from OFS.SimpleItem import SimpleItem
from Products.Silva.helpers import add_and_edit
from Products.Silva.SilvaPermissions import ViewManagementScreens
from AccessControl import ClassSecurityInfo
import Globals
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.SilvaStatic.httrackharvester import HttrackHarvester
from Products.SilvaStatic.wgetharvester import WgetHarvester
from Products.SilvaStatic.ftptransport import FtpTransport

class StaticService(SimpleItem):
    security = ClassSecurityInfo()
    meta_type = 'Silva Static Upload Service'

    manage_options = ({'label': 'Edit', 'action': 'edit_tab'},
                        ) + SimpleItem.manage_options

    security.declareProtected(ViewManagementScreens, 'edit_tab')
    security.declareProtected(ViewManagementScreens, 'manage_main')
    manage_main = edit_tab = PageTemplateFile(
        'www/serviceStaticEdit',
        globals(), __name__='edit_tab')
    
    def __init__(self, id, title):
        self.id = id
        self.title = title
        self._hostname = ''
        self._username = ''
        self._password = ''
        self._target_directory = ''
        self._harvester = 'httrack'
        self._harvest_root = ''
        
    security.declareProtected(ViewManagementScreens, 'manage_edit')
    def manage_edit(self, hostname, username, password, target_directory,
                    harvester, harvest_root):
        """Edit parameters to FTP upload.
        """
        self._hostname = hostname
        self._username = username
        self._harvester = harvester
        if password:
            self._password = password
        self._target_directory = target_directory
        self._harvest_root = harvest_root.strip()
        return self.edit_tab(
            manage_tabs_message="Configured FTP upload parameters")

    security.declarePrivate('staticize')
    def staticize(self):
        """Given a harvester and a transport, make static a site.
        """
        harvester = self.getHarvester()
        harvester.harvest()
        transport = self.getTransport()
        transport.upload(harvester.getDirectory())
        harvester.close()
        transport.close()
        
    security.declarePrivate('getTransport')
    def getTransport(self):
        return FtpTransport(self._hostname, self._username, self._password,
                            self._target_directory)

    security.declarePrivate('getHarvester')
    def getHarvester(self):
        if self._harvester == 'httrack':
            return HttrackHarvester(self.getRootUrl())
        elif self._harvester == 'wget':
            return WgetHarvester(self.getRootUrl())
        else:
            raise KeyError("Unknown harvester type: %s" % harvester)
        
    security.declarePrivate('getRootUrl')
    def getRootUrl(self):
        if self._harvest_root:
            return self._harvest_root
        return self.aq_inner.aq_parent.absolute_url()  

    security.declareProtected(ViewManagementScreens, 'editInfo')
    def editInfo(self):
        """Get information to display edit screen."""
        return {
            'hostname': self._hostname,
            'username': self._username,
            'target_directory': self._target_directory,
            'harvester': self._harvester,
            'harvest_root': self._harvest_root,
            }
        
Globals.InitializeClass(StaticService)

manage_addStaticServiceForm = PageTemplateFile(
    'www/serviceStaticAdd', globals(),
    __name__ = 'manage_addStaticServiceForm')

def manage_addStaticService(self, id, title, REQUEST=None):
    """Add a service."""
    id = self._setObject(id, StaticService(id, title))
    add_and_edit(self, id, REQUEST)
    return ''
